@@ -64,16 +64,17 @@ panel_util_launch_from_key_file (GKeyFile   *keyfile,
 				       "Exec",
 				       "URL",
 				       "Dev",
-				       "Name",
-				       "GenericName",
-				       "Icon",
-				       "MiniIcon",
 				       "Path",
 				       "Terminal",
 				       "TerminalOptions",
 				       "StartupNotify",
 				       "StartupWMClass",
 				     };
+	static const char *locale_keys [] = { "Name",
+					      "GenericName",
+					      "Icon",
+					      "MiniIcon",
+					    };
 
 	g_return_if_fail (keyfile != NULL);
 
@@ -84,14 +85,25 @@ panel_util_launch_from_key_file (GKeyFile   *keyfile,
 	for (i = 0; i < G_N_ELEMENTS (keys); i++) {
 		char *value;
 
-		value = g_key_file_get_value (keyfile, "Desktop Entry",
-					      keys [i], NULL);
+		value = panel_util_key_file_get_string (keyfile,
+							keys [i]);
 		if (value != NULL) {
 			gnome_desktop_item_set_string (ditem, keys [i], value);
 			g_free (value);
 		}
 	}
 
+	for (i = 0; i < G_N_ELEMENTS (locale_keys); i++) {
+		char *value;
+
+		value = panel_util_key_file_get_locale_string (keyfile,
+							       locale_keys [i]);
+		if (value != NULL) {
+			gnome_desktop_item_set_string (ditem, locale_keys [i], value);
+			g_free (value);
+		}
+	}
+
 	panel_ditem_launch (ditem, file_list, screen, error);
 	gnome_desktop_item_unref (ditem);
 }
@@ -117,8 +129,10 @@ panel_launch_desktop_file (const char  *desktop_file,
 	} else if (fallback_exec != NULL) {
 		char *argv [2] = {(char *)fallback_exec, NULL};
 
-		if (*error)
+		if (*error) {
 			g_error_free (*error);
+			*error = NULL;
+		}
 
 		gdk_spawn_on_screen (screen, NULL, argv, NULL,
 				     G_SPAWN_SEARCH_PATH,
@@ -126,6 +140,37 @@ panel_launch_desktop_file (const char  *desktop_file,
 	}
 }
 
+char *
+panel_util_make_exec_uri_for_desktop (const char *exec)
+{
+	GString    *str;
+	const char *c;
+
+	if (!exec)
+		return g_strdup ("");
+
+	if (!strchr (exec, ' '))
+		return g_strdup (exec);
+
+	str = g_string_new_len (NULL, strlen (exec));
+
+	str = g_string_append_c (str, '"');
+	for (c = exec; *c != '\0'; c++) {
+		/* FIXME: GKeyFile will add an additional backslach so we'll
+		 * end up with toto\\" instead of toto\"
+		 * We could use g_key_file_set_value(), but then we don't
+		 * benefit from the other escaping that glib is doing...
+		 */
+		if (*c == '"')
+			str = g_string_append (str, "\\\"");
+		else
+			str = g_string_append_c (str, *c);
+	}
+	str = g_string_append_c (str, '"');
+
+	return g_string_free (str, FALSE);
+}
+
 void
 panel_show_help (GdkScreen  *screen,
 		 const char *doc_name,
@@ -327,19 +372,27 @@ panel_error_dialog (GtkWindow  *parent,
 	}
 
 	dialog = gtk_message_dialog_new (parent, 0, GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK, primary_text);
+					 GTK_BUTTONS_OK, "%s", primary_text);
 	if (secondary_text != NULL)
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  secondary_text);
+							  "%s", secondary_text);
 
 	gtk_widget_add_events (dialog, GDK_KEY_PRESS_MASK);
 	g_signal_connect (dialog, "event",
 			  G_CALLBACK (panel_dialog_window_event), NULL);
 
-	gtk_window_set_wmclass (GTK_WINDOW (dialog), class, "Panel");
 	if (screen)
 		gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
+	if (!parent) {
+		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+		/* FIXME: We need a title in this case, but we don't know what
+		 * the format should be. Let's put something simple until
+		 * the following bug gets fixed:
+		 * http://bugzilla.gnome.org/show_bug.cgi?id=165132 */
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
+	}
+
 	gtk_widget_show_all (dialog);
 
 	if (auto_destroy)
@@ -676,39 +729,6 @@ panel_load_icon (GtkIconTheme  *icon_theme,
 	return retval;
 }
 
-GdkPixbuf *
-missing_pixbuf (int size)
-{
-	int i, j;
-	int rowstride;
-	guchar *pixels;
-	GdkPixbuf *pb;
-
-	pb = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-			     FALSE /* has alpha */,
-			     8 /* bits per sample */,
-			     size, size);
-	pixels = gdk_pixbuf_get_pixels (pb);
-	rowstride = gdk_pixbuf_get_rowstride (pb);
-
-	for (i = 0; i < size; i++) {
-		for (j = 0; j < size; j++) {
-			guchar *p = pixels + i * rowstride + 3 * j;
-			if (i == j || size - i == j) {
-				p[0] = 255;
-				p[1] = 0;
-				p[2] = 0;
-			} else {
-				p[0] = 255;
-				p[1] = 255;
-				p[2] = 255;
-			}
-		}
-	}
-
-	return pb;
-}
-
 static char *
 panel_lock_screen_action_get_command (const char *action)
 {
@@ -731,10 +751,16 @@ panel_lock_screen_action_get_command (const char *action)
 		}
 	} else if (strcmp (action, "activate") == 0
 		   || strcmp (action, "lock") == 0) {
-		if (use_gscreensaver) {
-			command = g_strdup_printf ("gnome-screensaver-command --%s", action);
+		/* Neither gnome-screensaver or xscreensaver allow root
+		 * to lock the screen */
+		if (geteuid () == 0) {
+			command = NULL;
 		} else {
-			command = g_strdup_printf ("xscreensaver-command -%s", action);
+			if (use_gscreensaver) {
+				command = g_strdup_printf ("gnome-screensaver-command --%s", action);
+			} else {
+				command = g_strdup_printf ("xscreensaver-command -%s", action);
+			}
 		}
 	}
 
@@ -821,53 +847,99 @@ panel_make_full_path (const char *dir,
 }
 
 char *
-panel_make_unique_uri (const char *dir,
-		       const char *suffix)
+panel_make_unique_desktop_path_from_name (const char *dir,
+					  const char *name)
 {
-#define NUM_OF_WORDS 12
-	char *words[] = {
-		"foo",
-		"bar",
-		"blah",
-		"gegl",
-		"frobate",
-		"hadjaha",
-		"greasy",
-		"hammer",
-		"eek",
-		"larry",
-		"curly",
-		"moe",
-		NULL};
+	int   num = 1;
+	char *path = NULL;
+	char  filename[NAME_MAX];
+
+/* g_file_set_contents() use "%s.XXXXXX"
+ * FIXME: waiting for http://bugzilla.gnome.org/show_bug.cgi?id=437977 */
+#define LENGTH_FOR_TMPFILE_EXT 7
+
+	g_snprintf (filename,
+		    sizeof (filename) - strlen (".desktop") - LENGTH_FOR_TMPFILE_EXT,
+		    "%s", name);
+	g_strlcat (filename, ".desktop", sizeof (filename));
+	path = panel_make_full_path (dir, filename);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS))
+		return path;
+	g_free (path);
+
+	while (TRUE) {
+		char *buf;
+
+		buf = g_strdup_printf ("-%d.desktop", num);
+		g_snprintf (filename,
+			    sizeof (filename) - strlen (buf) - LENGTH_FOR_TMPFILE_EXT,
+			    "%s", name);
+		g_strlcat (filename, buf, sizeof (filename));
+		g_free (buf);
+
+		path = panel_make_full_path (dir, filename);
+		if (!g_file_test (path, G_FILE_TEST_EXISTS))
+			return path;
+		g_free (path);
+
+		num++;
+	}
+
+	return NULL;
+}
+
+char *
+panel_make_unique_desktop_uri (const char *dir,
+			       const char *source)
+{
+	char     *name, *p;
 	char     *uri;
 	char     *path = NULL;
-	gboolean  exists = TRUE;
 
-	while (exists) {
-		char *filename;
-		int   rnd;
-		int   word;
+	/* Accept NULL source. Using an emptry string makes our life easier
+	 * than keeping NULL. */
+	if (!source)
+		source = "";
 
-		rnd = g_random_int ();
-		word = g_random_int () % NUM_OF_WORDS;
-
-		filename = g_strdup_printf ("%s-%010x%s",
-					    words [word],
-					    (guint) rnd,
-					    suffix);
-
-		g_free (path);
-		path = panel_make_full_path (dir, filename);
-		exists = g_file_test (path, G_FILE_TEST_EXISTS);
-		g_free (filename);
+	/* source may be an exec string, a path, or a URI. We truncate
+	 * it at the first space (to get just the command name if it's
+	 * an exec string), strip the path/URI, and remove the suffix
+	 * if it's ".desktop".
+	 */
+	name = g_strndup (source, strcspn (source, " "));
+	p = strrchr (name, '/');
+	while (p && !*(p + 1)) {
+		*p = '\0';
+		p = strrchr (name, '/');
 	}
+	if (p)
+		memmove (name, p + 1, strlen (p + 1) + 1);
+	p = strrchr (name, '.');
+	if (p && !strcmp (p, ".desktop")) {
+		*p = '\0';
+
+		/* also remove the -%d that might be at the end of the name */
+		p = strrchr (name, '-');
+		if (p) {
+			char *end;
+			strtol ((p + 1), &end, 10);
+			if (!*end)
+				*p = '\0';
+		}
+	}
+
+	if (name[0] == '\0') {
+		g_free (name);
+		name = g_strdup (_("file"));
+	}
+
+	path = panel_make_unique_desktop_path_from_name (dir, name);
+	g_free (name);
 
 	uri = gnome_vfs_get_uri_from_local_path (path);
 	g_free (path);
 
 	return uri;
-
-#undef NUM_OF_WORDS
 }
 
 static char *
@@ -1203,3 +1275,51 @@ panel_util_key_file_remove_locale_key (GKeyFile    *keyfile,
 		g_key_file_remove_key (keyfile, "Desktop Entry",
 				       key, NULL);
 }
+
+void
+panel_util_key_file_ensure_C_key (GKeyFile   *keyfile,
+				  const char *key)
+{
+	char *C_value;
+	char *buffer;
+
+	/* Make sure we set the "C" locale strings to the terms we set here.
+	 * This is so that if the user logs into another locale they get their
+	 * own description there rather then empty. It is not the C locale
+	 * however, but the user created this entry herself so it's OK */
+	C_value = panel_util_key_file_get_string (keyfile, key);
+	if (C_value == NULL || C_value [0] == '\0') {
+		buffer = panel_util_key_file_get_locale_string (keyfile, key);
+		if (buffer) {
+			panel_util_key_file_set_string (keyfile, key, buffer);
+			g_free (buffer);
+		}
+	}
+	g_free (C_value);
+}
+
+char *
+guess_icon_from_exec (GtkIconTheme *icon_theme,
+		      GKeyFile     *key_file)
+{
+	char *exec;
+	char *icon_name;
+	char *path;
+
+	exec = panel_util_key_file_get_string (key_file, "Exec");
+	if (!exec || !exec [0]) {
+		g_free (exec);
+		return NULL;
+	}
+
+	icon_name = g_path_get_basename (exec);
+	g_free (exec);
+
+	path = panel_find_icon (icon_theme, icon_name, 48);
+	if (!path) {
+		g_free (icon_name);
+		return NULL;
+	}
+
+	return icon_name;
+}
